import base64
import hashlib

import ecdsa


def webhook_authentication(pub_key_base64: str, x_sign_base64: str, response: str) -> bool:
    pub_key_bytes = base64.b64decode(pub_key_base64)
    signature_bytes = base64.b64decode(x_sign_base64)
    pub_key = ecdsa.VerifyingKey.from_pem(pub_key_bytes.decode())

    body_bytes = bytes(response, 'utf-8')

    if pub_key.verify(signature_bytes, body_bytes, sigdecode=ecdsa.util.sigdecode_der, hashfunc=hashlib.sha256):
        return True

    return False



from monopay.utils import to_camel_case


def data_builder(**kwargs):
    data = {}

    for kwarg in kwargs:
        key = kwarg.replace("timestamp", "")
        key = to_camel_case(key)
        value = kwargs.get(kwarg)
        data[key] = value

    return data



from typing import Union
from ..responses import QrDetailsResponse, QrListResponse, EmptyResponse
from ..exceptions import MonoPayBaseException
from ..instances import QrListItem
from ..utils import is_exception
from ..decorators import request, post_request
import requests


class Qr:
    def __init__(
        self,
        base_url: str,
        headers
    ):
        self.base_url = base_url
        self.headers = headers

        self.qr_list_url = f"{self.base_url}/api/merchant/qr/list"
        self.qr_details_url = f"{self.base_url}/api/merchant/qr/details"
        self.qr_reset_amount_url = f"{self.base_url}/api/merchant/qr/reset-amount"

    @request(url='qr_list_url')
    def list(self, response_data) -> Union[QrListResponse, MonoPayBaseException]:

        items = [
            QrListItem(
                short_qr_id=item['shortQrId'],
                qr_id=item['qrId'],
                amount_type=item['amountType'],
                page_url=item['pageUrl']
            ) for item in response_data['list']
        ]

        return QrListResponse(
            list=items
        )

    @request(url='qr_details_url')
    def details(
        self,
        qr_id: str,
        response_data
    ) -> Union[QrDetailsResponse, MonoPayBaseException]:

        return QrDetailsResponse(
            short_qr_id=response_data['shortQrId'],
            invoice_id=response_data['invoiceId'] if "invoiceId" in response_data else None,
            amount=response_data['amount'] if "amount" in response_data else None,
            ccy=response_data['ccy'] if "ccy" in response_data else None,
        )

    @post_request(url='qr_reset_amount_url')
    def reset_amount(
        self,
        qr_id: str,
        response_data
    ) -> Union[EmptyResponse, MonoPayBaseException]:

        return EmptyResponse()




from typing import Union, Optional
from monopay.responses import MerchantDetailsResponse, MerchantStatementResponse, MerchantPubKeyResponse
from monopay.utils import is_exception
from monopay.exceptions import MonoPayBaseException
from monopay.instances import MerchantStatementItem, CancelListItem
from monopay.decorators import request
import requests


class Merchant:
    def __init__(self, base_url: str, headers):
        self.base_url = base_url
        self.headers = headers

        self.merchant_details_url = f"{self.base_url}/api/merchant/details"
        self.merchant_statement_url = f"{self.base_url}/api/merchant/statement"
        self.merchant_pubkey_url = f"{self.base_url}/api/merchant/pubkey"

    @request(url='merchant_details_url')
    def details(self, response_data) -> Union[MerchantDetailsResponse, MonoPayBaseException]:

        return MerchantDetailsResponse(
            merchant_id=response_data['merchantId'],
            merchant_name=response_data['merchantName'],
        )

    @request('merchant_statement_url')
    def statement(
        self,
        from_timestamp: int,
        to_timestamp: Optional[int] = None,
        response_data=None,
    ) -> Union[MerchantStatementResponse, MonoPayBaseException]:

        return MerchantStatementResponse(
            list=[
                MerchantStatementItem(
                    invoice_id=statement_item['invoiceId'],
                    status=statement_item['status'],
                    masked_pan=statement_item['maskedPan'],
                    date=statement_item['date'],
                    payment_scheme=statement_item['paymentScheme'],
                    amount=statement_item['amount'],
                    profit_amount=statement_item['profitAmount'],
                    ccy=statement_item['ccy'],
                    approval_code=statement_item['approvalCode'],
                    rrn=statement_item['rrn'],
                    reference=statement_item['reference'],
                    short_qr_id=statement_item['shortQrId'],
                    cancel_list=[
                        CancelListItem(
                            amount=cancel_item['amount'],
                            ccy=cancel_item['ccy'],
                            date=cancel_item['date'],
                            approval_code=cancel_item['approvalCode'],
                            rrn=cancel_item['rrn'],
                            masked_pan=cancel_item['maskedPan']
                        ) for cancel_item in statement_item['cancelList']
                    ]
                ) for statement_item in response_data['list']
            ]
        )

    @request(url='merchant_pubkey_url')
    def pubkey(self, response_data) -> Union[MerchantPubKeyResponse, MonoPayBaseException]:

        return MerchantPubKeyResponse(
            key=response_data['key']
        )



import json
from typing import Union, Optional
from ..utils import is_exception
from ..exceptions import MonoPayBaseException
from ..responses import InvoiceCreatedResponse, SplitInvoiceResponse, InvoiceCanceledResponse, \
    InvoiceStatusResponse, FinalizeInvoiceResponse, EmptyResponse
import requests


class Invoice:
    def __init__(self, base_url, headers):
        self.base_url = base_url
        self.headers = headers

        self.create_invoice_url = f"{self.base_url}/api/merchant/invoice/create"
        self.split_invoice_url = f"{self.base_url}/api/merchant/invoice/split-payments"
        self.cancel_invoice_url = f"{self.base_url}/api/merchant/invoice/cancel"
        self.invoice_status_url = f"{self.base_url}/api/merchant/invoice/status"
        self.invoice_invalidation_url = f"{self.base_url}/api/merchant/invoice/remove"
        self.invoice_info_url = f"{self.base_url}/api/merchant/invoice/payment-info"
        self.invoice_finalize_url = f"{self.base_url}/api/merchant/invoice/finalize"

    def create(
        self,
        amount: int,
        ccy: Optional[int] = None,
        merchant_payment_info: Optional = None,
        redirect_url: Optional[str] = None,
        web_hook_url: Optional[str] = None,
        validity: Optional[int] = None,
        payment_type: Optional[str] = None,
        qr_id: Optional = None,
        save_card_data: Optional = None,
    ) -> Union[InvoiceCreatedResponse, MonoPayBaseException]:
        data = {
            "amount": amount,
            "ccy": ccy,
            "merchantPaymentInfo": merchant_payment_info,
            "redirectUrl": redirect_url,
            "webHookUrl": web_hook_url,
            "validity": validity,
            "paymentType": payment_type,
            "qrId": qr_id,
            "saveCardData": save_card_data
        }
        response = requests.post(
            url=self.create_invoice_url,
            headers=self.headers,
            data=json.dumps(data),
        )

        response_data = response.json()

        if is_exception(response):
            return MonoPayBaseException(
                err_code=response_data['errCode'],
                err_text=response_data['errText']
            )

        return InvoiceCreatedResponse(
            invoice_id=response_data['invoiceId'],
            page_url=response_data['pageUrl']
        )

    def split(self) -> Union[SplitInvoiceResponse, MonoPayBaseException]:
        response = requests.post(
            url=self.split_invoice_url,
            headers=self.headers,
        )

        response_data = response.json()

        if is_exception(response):
            return MonoPayBaseException(
                err_code=response_data['errCode'],
                err_text=response_data['errText']
            )

        return SplitInvoiceResponse(
            reference=response_data['reference']
        )

    def cancel(
        self,
        invoice_id: str,
        ext_ref: str,
        amount: int,
        items
    ) -> Union[InvoiceCanceledResponse, MonoPayBaseException]:
        data = {
            "invoiceId": invoice_id,
            "extRef": ext_ref,
            "amount": amount,
            "items": items
        }
        response = requests.post(
            url=self.cancel_invoice_url,
            headers=self.headers,
            data=data
        )

        response_data = response.json()

        if is_exception(response):
            return MonoPayBaseException(
                err_code=response_data['errCode'],
                err_text=response_data['errText']
            )

        return InvoiceCanceledResponse(
            status=response_data['status'],
            created_date=response_data['createdDate'],
            modified_date=response_data['modifiedDate']
        )

    def invalidation(
        self,
        invoice_id: str
    ) -> Union[MonoPayBaseException, EmptyResponse]:
        data = {
            "invoiceId": invoice_id
        }
        response = requests.post(
            url=self.invoice_invalidation_url,
            headers=self.headers,
            data=data
        )

        response_data = response.json()

        if is_exception(response):
            return MonoPayBaseException(
                err_code=response_data['errCode'],
                err_text=response_data['errText']
            )

        return EmptyResponse()

    def status(
        self,
        invoice_id: str
    ) -> Union[InvoiceStatusResponse, MonoPayBaseException]:

        params = {
            "invoiceId": invoice_id,
        }

        response = requests.get(
            url=self.invoice_status_url,
            headers=self.headers,
            params=params
        )

        response_data = response.json()

        if is_exception(response):
            return MonoPayBaseException(
                err_code=response_data['errCode'],
                err_text=response_data['errText']
            )

        return InvoiceStatusResponse()

    def info(
        self,
        invoice_id
    ):
        params = {
            "invoiceId": invoice_id,
        }
        response = requests.get(
            url=self.invoice_info_url,
            headers=self.headers,
            params=params
        )
        return response.json()

    def finalize(
        self,
        invoice_id: str,
        amount: int
    ) -> Union[FinalizeInvoiceResponse, MonoPayBaseException]:
        data = {
            "invoiceId": invoice_id,
            "amount": amount,
        }

        response = requests.post(
            url=self.invoice_finalize_url,
            headers=self.headers,
            data=data
        )

        response_data = response.json()

        if is_exception(response):
            return MonoPayBaseException(
                err_code=response_data['errCode'],
                err_text=response_data['errText']
            )

        return FinalizeInvoiceResponse(
            status=response_data['status']
        )




from typing import Union
from ..exceptions import MonoPayBaseException
from ..responses import EmptyResponse, WalletCardsResponse
from ..instances import WalletItem
from ..utils import is_exception
import requests


class Wallet:
    def __init__(
        self,
        base_url: str,
        headers
    ):
        self.base_url = base_url
        self.headers = headers

        self.cards_list_url = f"{self.base_url}/api/merchant/wallet"
        self.card_payment_url = f"{self.base_url}/api/merchant/wallet/payment"
        self.delete_card_url = f"{self.base_url}/api/merchant/wallet/card"

    def cards(self, wallet_id: str):
        params = {
            "walletId": wallet_id,
        }

        response = requests.get(
            url=self.cards_list_url,
            headers=self.headers,
            params=params
        )

        response_data = response.json()

        if is_exception(response):
            return MonoPayBaseException(
                err_code=response_data['errCode'],
                err_text=response_data['errText']
            )

        return WalletCardsResponse(
            wallets=[
                WalletItem(
                    card_token=item['cardToken'],
                    masked_pan=item['maskedPan'],
                    country=item['country']
                ) for item in response_data['wallets']
            ]
        )

    def payment(self):
        pass

    def delete_card(self, card_token: str) -> Union[EmptyResponse, MonoPayBaseException]:
        params = {
            "cardToken": card_token,
        }

        response = requests.delete(
            url=self.cards_list_url,
            headers=self.headers,
            params=params
        )

        response_data = response.json()

        if is_exception(response):
            return MonoPayBaseException(
                err_code=response_data['errCode'],
                err_text=response_data['errText']
            )

        return EmptyResponse()



from .__main__ import *
from . import exceptions
from . import utils
from . import instances
from . import responses
from . import methods

from .exceptions import *
from .utils import *
from .instances import *
from .responses import *
from .methods import *

__author__ = "yeghorkikhai"
__version__ = "0.1.4"
__email__ = "yeghorkikhai@gmail.com"



from .methods import Merchant, Invoice, Qr, Wallet


class AsyncMonoPay:
    def __init__(self, api_token):
        self.api_token = api_token
        self.base_url = "https://api.monobank.ua"
        self.headers = {
            "X-Token": self.api_token
        }
        self.merchant: Merchant = Merchant(
            base_url=self.base_url,
            headers=self.headers
        )
        self.invoice: Invoice = Invoice(
            base_url=self.base_url,
            headers=self.headers
        )
        self.qr: Qr = Qr(
            base_url=self.base_url,
            headers=self.headers
        )
        self.wallet: Wallet = Wallet(
            base_url=self.base_url,
            headers=self.headers
        )

    def set_api_token(self, api_token) -> None:
        self.api_token = api_token

    def get_api_token(self) -> str:
        return self.api_token



__all__ = [
    'InvoiceCreatedResponse',
    'InvoiceCanceledResponse',
    'InvoiceStatusResponse',
    'InvoiceInfoResponse',
    'SplitInvoiceResponse',
    'FinalizeInvoiceResponse',
    'QrListResponse',
    'QrDetailsResponse',
    'MerchantDetailsResponse',
    'MerchantStatementResponse',
    'MerchantPubKeyResponse',
    'WalletCardsResponse',
    'EmptyResponse'
]

from typing import Optional, List
from dataclasses import dataclass
from ..instances import QrListItem, MerchantStatementItem, WalletItem


@dataclass
class EmptyResponse:
    pass


@dataclass
class InvoiceCreatedResponse:
    invoice_id: str
    page_url: str


@dataclass
class InvoiceCanceledResponse:
    status: str
    created_date: str
    modified_date: str


@dataclass
class SplitInvoiceResponse:
    reference: str


@dataclass
class InvoiceStatusResponse:
    pass


@dataclass
class InvoiceInfoResponse:
    pass


@dataclass
class FinalizeInvoiceResponse:
    status: str


@dataclass
class QrListResponse:
    list: List[QrListItem]


@dataclass
class QrDetailsResponse:
    short_qr_id: str
    invoice_id: Optional[str]
    amount: Optional[int]
    ccy: Optional[int]


@dataclass
class MerchantDetailsResponse:
    merchant_id: str
    merchant_name: str


@dataclass
class MerchantStatementResponse:
    list: List[MerchantStatementItem]


@dataclass
class MerchantPubKeyResponse:
    key: str


@dataclass
class WalletCardsResponse:
    wallets: List[WalletItem]
